#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === HardwareInformation
#
# This class will show some information about the local computer system,
# e. g. hardware information, via colours.
# =========================================================================== #
# require 'hardware_information'
# =========================================================================== #
class HardwareInformation # === HardwareInformation

  require 'hardware_information/version/version.rb'
  require 'hardware_information/project/project.rb'

  # ========================================================================= #
  # === N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === ERROR_LINE
  # ========================================================================= #
  ERROR_LINE = '2>&1'

  # ========================================================================= #
  # === LEFT_PADDING
  #
  # How many characters to pad on the left side. Used for the
  # header_content() method.
  # ========================================================================= #
  LEFT_PADDING = 26

  # ========================================================================= #
  # === MAIN_DEFAULT_RJUST
  # ========================================================================= #
  MAIN_DEFAULT_RJUST = 48

  # ========================================================================= #
  # === CPUINFO_LOCATION
  #
  # This constants stores where cpuinfo is kept. Will work only on
  # Linux, I think.
  # ========================================================================= #
  CPUINFO_LOCATION = '/proc/cpuinfo'

  # ========================================================================= #
  # === PROC_MEMINFO
  # ========================================================================= #
  PROC_MEMINFO = '/proc/meminfo'

  # ========================================================================= #
  # === FILE_COLOURS_FOR_THE_HARDWARE_PROJECT
  # ========================================================================= #
  FILE_COLOURS_FOR_THE_HARDWARE_PROJECT =
    "#{PROJECT_YAML_DIRECTORY}colours_for_the_hardware_information_project.yml"

  # ========================================================================= #
  # === HardwareInformation.kb_to_mb(i)
  # ========================================================================= #
  def self.kb_to_mb(i)
    (
      i.gsub(/ kB/,'').to_i / 1024
    ).to_s+ ' MB'
  end

  # ========================================================================= #
  # === HardwareInformation.is_on_windows?
  # ========================================================================= #
  def self.is_on_windows?
    if RUBY_PLATFORM.include?('win') or # if on windows.
       RUBY_PLATFORM.include?('mingw')
      return true
    end
    return false
  end

  # ========================================================================= #
  # === LSCPU
  #
  # The `lscpu` command may fail, which is why it has to be put into
  # a begin/rescue clause.
  # ========================================================================= #
  if is_on_windows?
    LSCPU = nil
  else
    begin
      LSCPU = `lscpu`.split("\n")
    rescue Errno::ENOENT
      puts 'No binary called lscpu could be found.'
    end
  end

  # ========================================================================= #
  # === CPUINFO
  #
  # We must first check whether the target location actually does happen
  # to exist.
  # ========================================================================= #
  _ = CPUINFO_LOCATION
  if File.exist? _
    CPUINFO = File.read(_)
  else
    CPUINFO = nil
  end

  # ========================================================================= #
  # === SPLITTED_CPUINFO
  # ========================================================================= #
  if CPUINFO
    SPLITTED_CPUINFO = CPUINFO.split(N)
    begin
      LSPCI = `lspci`
    rescue # Must rescue here because lspci might not be available.
      LSPCI = nil
    end
  end

  # ========================================================================= #
  # === DATASET_FROM_PROC_MEMINFO
  # ========================================================================= #
  if File.exist? PROC_MEMINFO
    DATASET_FROM_PROC_MEMINFO = File.readlines(PROC_MEMINFO)
    _ = DATASET_FROM_PROC_MEMINFO.select {|entry|
      entry.include? 'MemTotal'
    }.first.strip.split(':').last.strip
    # ======================================================================= #
    # We discard all but the gigs for RAM.
    # ======================================================================= #
    _ = (_.gsub(/ kB/,'').to_i / 1024).to_s+ ' MB'
  else
    DATASET_FROM_PROC_MEMINFO = nil
  end

  # ========================================================================= #
  # === RAM_AVAILABLE
  # ========================================================================= #
  RAM_AVAILABLE = _

  if DATASET_FROM_PROC_MEMINFO
    _ = DATASET_FROM_PROC_MEMINFO.select {|entry|
        entry.include? 'MemFree'
      }.first.strip.split(':').last.strip #.gsub(/ kB/,'').to_i / 1024).to_s+ ' MB'
    # ======================================================================= #
    # === RAM_AVAILABLE_FREE
    # ======================================================================= #
    RAM_AVAILABLE_FREE = kb_to_mb(_)
  end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    case run_already
    when :do_not_run_yet
      run_already = false
    end
    set_run_already(
      run_already
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @string
    # ======================================================================= #
    @string = ''.dup
    # ======================================================================= #
    # === @default_colour
    # ======================================================================= #
    @default_colour = green?
    # ======================================================================= #
    # === @default_padding
    #
    # This variable denotes the leading padding. That padding will be
    # prepended to the output.
    # ======================================================================= #
    @default_padding = '  '.dup # 2 spaces since as of December 2022.
    # ======================================================================= #
    # === @use_colours
    # ======================================================================= #
    @use_colours = true # If true we use ANSI colours.
    # ======================================================================= #
    # === @use_colon_as_delimiter
    # ======================================================================= #
    @use_colon_as_delimiter = true # If true we use colon as delimiter.
    # ======================================================================= #
    # === @hash_use_these_colours
    #
    # Next we will load the internal .yml file, if it exists.
    # ======================================================================= #
    @hash_use_these_colours = {}
    if File.exist? FILE_COLOURS_FOR_THE_HARDWARE_PROJECT
      require 'yaml'
      @hash_use_these_colours.update(
        YAML.load_file(FILE_COLOURS_FOR_THE_HARDWARE_PROJECT)
      )
    else # else use a hardcoded approach.
      @hash_use_these_colours[:filesize_of_the_harddisc] = :orchid
      @hash_use_these_colours[:file_system_type] = :steelblue
    end
    set_commandline_mode # Default.
  end

  # ========================================================================= #
  # === make_line
  #
  # The first argument to this method should be the title that will
  # be displayed.
  #
  # The second argument should show the content towards that argument.
  # ========================================================================= #
  def make_line(
      title, content
    )
    title = title.dup if title.frozen?
    use_this_colour_for_change = bold_teal?
    use_this_colour_to_revert  = @default_colour # First a copy.
    title << ': ' if @use_colon_as_delimiter
    title   = default_ljust(title)
    content = default_rjust(content)
    # ======================================================================= #
    # === Handle blocks next
    #
    # The passed block can modify the variable called content. This
    # will happen after the various padding-operations above.
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when nil
        # Pas through.
      else # Else, use colours.
        if use_colours?
          content = COLOURS.send(yielded, content)
        end
      end
    end
    string = ''.dup
    string << use_this_colour_for_change if use_colours?
    string << title
    string << use_this_colour_to_revert if use_colours?
    string = string.ljust(40)
    if content.is_a? Array
      _ string
      content.each {|line| _ "    #{line}" }
    else
      string << content
      _ string
    end
  end

  # ========================================================================= #
  # === start_the_sinatra_interface
  #
  # To invoke this method, do:
  #
  #   hardi --sinatra
  #
  # ========================================================================= #
  def start_the_sinatra_interface
    require 'hardware_information/www/my_hardware_for_sinatra.rb'
    HardwareInformation::SinatraMyHardware.start_sinatra_interface
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  #
  # Commandline-arguments, e. g. ARGV, will be passed into this menu
  # interface.
  # ========================================================================= #
  def menu(
      i = nil
    )
    if i
      if i.is_a? Array
        i.each {|entry| menu(entry) }
      else
        case i # case tag
        # =================================================================== #
        # === hardi --sinatra
        # =================================================================== #
        when /^-?-?sinatra$/
          start_the_sinatra_interface
        # =================================================================== #
        # === hardi --disable-colours
        # =================================================================== #
        when /^-?-?disable(-|_)?colou?rs$/,
             /^-?-?no(-|_)?colou?rs$/
          disable_colours
        end
      end
    end
  end

  # ========================================================================= #
  # === set_run_already
  # ========================================================================= #
  def set_run_already(i)
    @run_already = i
  end

  # ========================================================================= #
  # === pad?
  # ========================================================================= #
  def pad?
    @default_padding
  end

  # ========================================================================= #
  # === append_newline
  # ========================================================================= #
  def append_newline
    add N
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    e string?
  end

  # ========================================================================= #
  # === string?
  # ========================================================================= #
  def string?
    @string
  end; alias string string? # === string

  # ========================================================================= #
  # === set_commandline_mode
  # ========================================================================= #
  def set_commandline_mode
    @mode = :commandline
  end

  # ========================================================================= #
  # === set_web_mode
  # ========================================================================= #
  def set_web_mode
    @mode = :web
  end

  # ========================================================================= #
  # === default_ljust
  # ========================================================================= #
  def default_rjust(i)
    if i.is_a? Array # We do not handle Arrays.
      i
    else
      i.to_s.rjust(MAIN_DEFAULT_RJUST)
    end
  end

  # ========================================================================= #
  # === _                                                             (_ tag)
  #
  # This method, as a shortcut, makes use of the method called add(),
  # but it will also append a newline to the given input and support
  # colours (if given as argument).
  # ========================================================================= #
  def _(i, optional_colour = nil)
    if optional_colour and use_colours? # Add colours only if needed.
      if optional_colour.is_a? Symbol
        i = ::Colours.send(optional_colour, i)
      else
        i = optional_colour+i+rev
      end
    end
    add "#{i}#{N}" # We try it with mandatory newline for now.
  end

  # ========================================================================= #
  # === add_title
  # ========================================================================= #
  def add_title
    optional_colour = nil
    if use_colours?
      if Object.const_defined? :Colours # User has the colours gem installed.
        optional_colour = :mediumslateblue
      else
        optional_colour = green?
      end
    end
    _ "#{N}Hardware Information"\
      "                                                    "\
      "💻#{N}", optional_colour
  end

  # ========================================================================= #
  # === colourize_filesize_of_the_harddisc
  # ========================================================================= #
  def colourize_filesize_of_the_harddisc(i)
    COLOURS.send(
      @hash_use_these_colours[:filesize_of_the_harddisc],
      i
    )
  end

  # ========================================================================= #
  # === colourize_file_system_type
  # ========================================================================= #
  def colourize_file_system_type(i)
    COLOURS.send(
      @hash_use_these_colours[:file_system_type],
      i
    )
  end

  # ========================================================================= #
  # === colourize_n_percent_occupied
  # ========================================================================= #
  def colourize_n_percent_occupied(i)
    COLOURS.send(
      @hash_use_these_colours[:n_percent_occupied],
      i
    )
  end

  # ========================================================================= #
  # === colourize_highlighted_content
  # ========================================================================= #
  def colourize_highlighted_content(i)
    COLOURS.send(
      @hash_use_these_colours[:highlighted_content],
      i
    )
  end

  # ========================================================================= #
  # === colourize_via_regular_colour
  #
  # Do note that this is presently hardcoded to the "thistle" colour.
  # Unsure whether to allow the user more customization here, so for
  # the time being this remains hardcoded.
  # ========================================================================= #
  def colourize_via_regular_colour(i)
    COLOURS.send(
      :thistle,
      i
    )
  end

  # ========================================================================= #
  # === append_output_generated_via_lsblk
  #
  # Typical output generated via lsblk may be like this:
  #
  #   NAME   MAJ:MIN RM  SIZE RO TYPE MOUNTPOINT
  #   sda      8:0    0  2.7T  0 disk 
  #   ├─sda1   8:1    0    1M  0 part 
  #   └─sda2   8:2    0  2.7T  0 part /
  #   sr0     11:0    1 1024M  0 rom
  #
  # ========================================================================= #
  def append_output_generated_via_lsblk
    _ = ''.dup
    # ======================================================================== #
    # Next we will display/obtain more information, via df.
    # ======================================================================== #
    extended_information = `df -T`.split(N)
    result = `lsblk`.strip.split(N).map {|line| "    #{line}" }
    result.each {|entry|
      # ===================================================================== #
      # Modify the input.
      # ===================================================================== #
      if entry.include? ' /'
        # =================================================================== #
        # These entries will be colourized. Note that the first part may
        # look like this: "└─sda2"
        #
        #   "    └─sda2   8:2    0  2.7T  0 part /"
        #
        # =================================================================== #
        splitted = entry.split(' ')
        # =================================================================== #
        # The root_entry may look like this:
        #
        #   /dev/sda2      ext4     2883218608 529813416 2206875608  20% /
        #
        # =================================================================== #
        selection = extended_information.select {|line|
          last = line.split(' ').last
          last == '/' or last.include?('USB')
        }
        if selection
          selection = selection.select {|inner_line|
            inner_line.end_with? splitted[-1]
          }
          if selection.is_a?(Array) and selection.first
            selection = selection.first.split(' ')
          end
        end
        splitted[-1] = colourize_highlighted_content(splitted[-1].ljust(12)) # ← The last part.
        splitted[0] = colourize_highlighted_content("    #{splitted[0]}")    # ← The first part.
        splitted[1] = colourize_via_regular_colour(splitted[1].strip.ljust(4).rjust(6))
        splitted[2] = colourize_via_regular_colour(" #{splitted[2].strip.rjust(2)}")
        splitted[3] = colourize_filesize_of_the_harddisc(splitted[3].strip.rjust(5).ljust(6)) # ← The size of the harddisc.
        splitted[4] = colourize_via_regular_colour(splitted[4].strip.ljust(2).rjust(3))
        splitted[5] = colourize_via_regular_colour(splitted[5].strip)
        file_system_type   = selection[1]
        n_percent_occupied = selection[5]
        # =================================================================== #
        # /dev/sda2   ext4    2883218608 529753916 2206935108  20% /"]
        # =================================================================== #
        splitted = splitted.join(' ')
        unless entry.include? '/boot/' # ← Don't show information for /boot/ entries.
          splitted << colourize_file_system_type(" (#{file_system_type})")
          splitted << colourize_n_percent_occupied(" (#{n_percent_occupied} used)")
        end
        entry = splitted
      else # Else we only do a slightly simpler colourization-part.
        splitted = entry.split(' ')
        splitted[0] = colourize_via_regular_colour("    #{splitted[0].ljust(6)}")      # ← The first part.
        splitted[1] = colourize_via_regular_colour(splitted[1].strip.ljust(4).rjust(6))
        splitted[2] = colourize_via_regular_colour(" #{splitted[2].strip.rjust(2)}")
        splitted[3] = colourize_filesize_of_the_harddisc(splitted[3].strip.rjust(5).ljust(6)) # ← The size of the harddisc.
        splitted[4] = colourize_via_regular_colour(splitted[4].strip.ljust(2).rjust(3))
        splitted[5] = colourize_via_regular_colour(splitted[5].strip)
        if splitted[6] and (splitted[6] == '[SWAP]')
          splitted[6] = royalblue(splitted[6])
        end
        entry = splitted.join(' ')
        entry = colourize_via_regular_colour(entry)
      end
      _ << "#{entry}\n" # Now we can append this.
    }
    # ======================================================================= #
    # Add two newlines too, for slightly more readable output.
    # ======================================================================= #
    _ << "#{N}#{N}"
    add _
  end

  # ========================================================================= #
  # === default_ljust
  # ========================================================================= #
  def default_ljust(i)
    i.to_s.ljust(15)
  end
  begin
    require 'colours'
    include ::Colours
  rescue LoadError; end

  # ========================================================================= #
  # === COLOURS
  # ========================================================================= #
  COLOURS = ::Colours

  # ========================================================================= #
  # === green?
  # ========================================================================= #
  def green?
    COLOURS::GREEN
  end

  # ========================================================================= #
  # === bold_teal?
  # ========================================================================= #
  def bold_teal?
    COLOURS::BOLD_TEAL
  end

  # ========================================================================= #
  # === use_colours?
  # ========================================================================= #
  def use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === enable_colours
  # ========================================================================= #
  def enable_colours
    @use_colours = true
  end

  # ========================================================================= #
  # === disable_colours
  # ========================================================================= #
  def disable_colours
    @use_colours = false
  end

  # ========================================================================= #
  # === thistle
  # ========================================================================= #
  def thistle(i)
    return COLOURS.thistle(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === royalblue
  # ========================================================================= #
  def royalblue(i)
    return COLOURS.royalblue(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === steelblue
  # ========================================================================= #
  def steelblue(i)
    return COLOURS.steelblue(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    if use_colours?
      COLOURS.rev
    else
      ''
    end
  end

  # ========================================================================= #
  # === add_audio_information                                     (audio tag)
  # ========================================================================= #
  def add_audio_information
    audio_device = `aplay -l`.scan(/card.+$/).first
    make_line(pad?+'Audio Device', N+(pad? * 2)+audio_device.to_s)
  end

  # ========================================================================= #
  # === find_out_the_amount_of_processors
  # ========================================================================= #
  def find_out_the_amount_of_processors
    data = CPUINFO.scan(/cpu cores\t: (\d+)\n/).flatten.map(&:to_i).sum.to_i
    make_line(pad?+'Processors', data) { :slateblue }
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  # ========================================================================= #
  def add(i)
    @string << i
  end

  require 'hardware_information/monitor/monitor.rb'
  # ========================================================================= #
  # === obtain_information_about_the_monitor
  # ========================================================================= #
  def obtain_information_about_the_monitor
    make_line(
      "#{pad?}Monitor resolution",
      ::HardwareInformation.return_the_monitor_resolution
    ) { :mediumspringgreen }
  end

  # ========================================================================= #
  # === obtain_information_about_the_wireless_card
  #
  # This method will display information about the wireless card (WLAN
  # device), should it exist.
  # ========================================================================= #
  def obtain_information_about_the_wireless_card
    result = `lspci #{ERROR_LINE}`
    if result and result.is_a?(String) and !result.empty?
      splitted = result.split("\n")
      selection = splitted.select {|entry|
        entry.include? 'Wireless'
      }
      if selection and !selection.empty?
        _ = selection.first.split(':').last.to_s
        make_line("#{pad?}WLAN", _) { :mediumspringgreen }
      end
    end
  end

  # ========================================================================= #
  # === obtain_virtualization
  #
  # This method call may fail, depending on the output of the LSCPU
  # command, hence why this check has to be a bit more complicated.
  # ========================================================================= #
  def obtain_virtualization
    result = LSCPU.grep(/Virtualization:/)
    if result
      result = result.first.split(':').last.strip
      make_line(pad?+'Virtualization', result) { :mediumseagreen }
    end
  end

  # ========================================================================= #
  # === obtain_information_about_the_GPU
  # ========================================================================= #
  def obtain_information_about_the_GPU
    make_line(pad?+'GPU',
      return_information_about_the_GPU
    ) { :mediumpurple }
  end

  # ========================================================================= #
  # === return_information_about_the_GPU
  #
  # This method will return information about the GPU in use on the given
  # computer system.
  #
  # This may be similar to a String such as:
  #
  #   GPU: AMD ATI Radeon R7 Graphics
  #
  # ========================================================================= #
  def return_information_about_the_GPU
    result = `lspci #{ERROR_LINE}`.strip
    if result and result.include?('VGA compatible controller')
      result = result.split(N).select {|line|
        line.include? 'VGA compatible controller'
      }.first
      # ===================================================================== #
      # Work on:
      #
      #   VGA compatible controller: Advanced Micro Devices, Inc. [AMD/ATI] Kaveri [Radeon R7 Graphics]"
      #
      # ===================================================================== #
      result = result.split('. [').last.
               delete('[]').tr('/',' ')
    end
    return result
  end

  # ========================================================================= #
  # === assumed_path_to_dmidecode?
  # ========================================================================= #
  def assumed_path_to_dmidecode?
    _ = '/usr/bin/dmidecode'
    unless File.exist? _
      if File.exist?('/usr/sbin/dmidecode')
        _ = '/usr/sbin/dmidecode'
      end
    end
    return _
  end

  # ========================================================================= #
  # === obtain_firmware_information
  #
  # Note that this functionality depends on the binary called "lshw".
  # ========================================================================= #
  def obtain_firmware_information
    result = `lshw #{ERROR_LINE}`
    if result.include? 'lshw: command not found'
      e '(Note that the binary '+steelblue('lshw')+
        ' could not be found.)'
    end
    if result and !result.include?('command not found')
      # ===================================================================== #
      # See: https://rubular.com/r/Uf4XqmbZW3atCy
      # ===================================================================== #
      regex = 
        /description: Motherboard\s+product: (.+)\s+vendor: (.+)\s+[.:0-9a-zA-Z\n ]+\*-firmware[.:0-9a-zA-Z\n ]+version: (\d{0,4}[0-9.]+)\s+date: (\d{1,2}.+)/
      result =~ regex
      motherboard = $1.to_s.dup
      motherboard_vendor = $2.to_s.dup
      bios_version = $3.to_s.dup
      bios_release = $4.to_s.dup
    end
    if (bios_version.nil? or bios_version.empty?) and result.include?('command not found')
      # ===================================================================== #
      # Try dmidecode in this case.
      # ===================================================================== #
      result = `#{assumed_path_to_dmidecode?} #{ERROR_LINE}`
      if result
        result = result.split("\n").select {|line| line.include? 'BIOS Revision: ' }
        if result and result.is_a?(Array)
          result = result.first.to_s
          bios_version = result.split(':').last.strip if result.include?(':')
        end
      end
    end
    # ======================================================================= #
    # Show some more information next, in particular the BIOS.
    # ======================================================================= #
    make_line(pad?+'Motherboard vendor', motherboard_vendor)
    make_line(pad?+'Motherboard',        motherboard)
    make_line(pad?+'BIOS version',       bios_version)
    make_line(pad?+'BIOS released on',   bios_release)
  end

  # ========================================================================= #
  # === obtain_vendor_id
  # ========================================================================= #
  def obtain_vendor_id
    result = LSCPU.grep(/Vendor ID:/).first.split(':').last.strip
    make_line(pad?+'Vendor ID', result) { :mediumseagreen }
  end

  # ========================================================================= #
  # === obtain_n_cores
  #
  # Return how many cores that machine has.
  # ========================================================================= #
  def obtain_n_cores
    make_line(pad?+'n CPU cores',
      `nproc --all`.strip
    ) { :lightgreen }
  end

  # ========================================================================= #
  # === obtain_ram                                                  (ram tag)
  # ========================================================================= #
  def obtain_ram
    make_line(pad?+'RAM (max)', RAM_AVAILABLE) { :orangered }
  end

  # ========================================================================= #
  # === obtain_ram_free
  # ========================================================================= #
  def obtain_ram_free
    make_line(pad?+'RAM (free)', RAM_AVAILABLE_FREE) { :orangered }
  end

  # ========================================================================= #
  # === obtain_swap_size                                           (swap tag)
  # ========================================================================= #
  def obtain_swap_size
    _ = DATASET_FROM_PROC_MEMINFO.grep(/SwapTotal:/).first.
        split('  ').last.strip.upcase
    make_line(pad?+'SwapTotal', _) { :springgreen }
  end

  # ========================================================================= #
  # === obtain_system_bogomips
  #
  # BogoMIPS measure CPU speed.
  # ========================================================================= #
  def obtain_system_bogomips
    make_line pad?+'BogoMIPS',
      SPLITTED_CPUINFO.select {|line|
        line.include? 'bogomips'
      }.first.split(':')[1].strip
  end

  # ========================================================================= #
  # === obtain_pci_information
  # ========================================================================= #
  def obtain_pci_information
    array_fetch_these_types = %w(
      SMBus Audio IDE VGA
      Memory Controller
    )
    results = []
    if LSPCI
      LSPCI.split(N).each {|line|
        array_fetch_these_types.each { |inner_line|
          results << line if line.include? inner_line
        }
      }
      make_line pad?+'PCI Devices', results
    end
  end

  # ========================================================================= #
  # === obtain_chip_mhz
  # ========================================================================= #
  def obtain_chip_mhz
    make_line pad?+'Chip MHzx',
      SPLITTED_CPUINFO.select {|line|
        line.include? 'cpu MHz'
      }.first.split(':')[1].strip
  end

  # ========================================================================= #
  # === obtain_cache_size
  # ========================================================================= #
  def obtain_cache_size
    make_line pad?+'Cache Size',
      SPLITTED_CPUINFO.select {|line|
        line.include? 'cache size'
      }.first.split(':')[1].strip
  end

  # ========================================================================= #
  # === obtain_cpu_model
  # ========================================================================= #
  def obtain_cpu_model
    result = SPLITTED_CPUINFO.select {|line|
      line.include? 'model name'
    }.first.split(':')[1].strip
    make_line pad?+'Model', result
  end

  # ========================================================================= #
  # === obtain_byte_order
  # ========================================================================= #
  def obtain_byte_order
    result = LSCPU.grep(/Byte Order:/).first.split(':').last.strip
    make_line pad?+'Byte Order', result
  end

  # ========================================================================= #
  # === obtain_cpu_family
  # ========================================================================= #
  def obtain_cpu_family
    result = LSCPU.grep(/CPU family:/).first.split(':').last.strip
    make_line pad?+'CPU family', result
  end

  # ========================================================================= #
  # === obtain_model
  # ========================================================================= #
  def obtain_model
    result = LSCPU.grep(/Model:/).first.split(':').last.strip
    make_line pad?+'Model', result
  end

  # ========================================================================= #
  # === obtain_cpu_op_modes
  # ========================================================================= #
  def obtain_cpu_op_modes
    result = LSCPU.grep(/CPU op-mode/).first.split(':').last.strip
    make_line(pad?+'CPU op-mode(s)', result)
  end

  # ========================================================================= #
  # === obtain_linux_kernel_version
  #
  # This method should return a String, such as '4.14.4'.
  # ========================================================================= #
  def obtain_linux_kernel_version
    result = `uname -r`.strip
    make_line(pad?+'Linux kernel version', result)
  end

  # ========================================================================= #
  # === obtain_architecture
  # ========================================================================= #
  def obtain_architecture
    result = LSCPU.grep(/Architecture:/).first.split(':').last.strip
    make_line(pad?+'Architecture', result) { :violet }
  end


  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    add_title
    obtain_architecture                        # === Architecture
    obtain_cpu_model                           # === The CPU model, such as "AMD"
    obtain_cpu_op_modes                        # === CPU op-modes
    obtain_n_cores                             # === n CPU cores
    find_out_the_amount_of_processors          # === Amount of processors
    obtain_vendor_id                           # === Vendor ID
    obtain_virtualization                      # === Virtualization
    obtain_linux_kernel_version                # === The Linux kernel version
    obtain_ram                                 # === Amount of RAM
    obtain_ram_free
    obtain_swap_size                           # === SwapTotal
    obtain_chip_mhz                            # === Chip MHzx
    obtain_cache_size
    obtain_system_bogomips                     # === BogoMIPS
    obtain_byte_order
    obtain_cpu_family
    obtain_model                               # === Model
    obtain_information_about_the_GPU           # === Show the GPU in use
    obtain_information_about_the_monitor       # === Gather information about the monitor (Monitor resolution)
    obtain_information_about_the_wireless_card # === Wireless card
    obtain_firmware_information                # === Firmware Information
    obtain_pci_information
    add_audio_information                      # === Show the audio device
    append_newline
    append_output_generated_via_lsblk
    report_result if @run_already # Last but not least, report the result.
  end

  # ========================================================================= #
  # === HardwareInformation.run
  # ========================================================================= #
  def self.run
    new
  end; self.instance_eval { alias report run } # === HardwareInformation.report

end

if __FILE__ == $PROGRAM_NAME
  _ = HardwareInformation.new(false)
  # _.disable_colours
  _.run
  _.report_result
end # hardi